#!/usr/bin/env python3
"""
histogram1.py
Histogram of data
by Joel R. Voss
June 21, 2005
Ported to python 3 Jan 21, 2015
"""
from __future__ import print_function
from sys import argv

def bufferBinaryHistogram(data, d=None):
	"""
	Returns a histogram of a string or a binary.
	[0,0,0,0,...,53,12,99,44,0,0,0,...]
	"""
	if d == None or len(d) == 0:
		d = [0 for i in range(256)]
	#end if
	
	for c in data:
		e = c
		if type(c) == str: e = ord(c)
		d[e] += 1
	#next c
	
	return d
#end def bufferBinaryHistogram(data, d=[])

def HistogramProbability(hist):
	"""
	Multiply a histogram's values by 256 and divide by sum(hist)
	"""
	output = []
	hsum = sum(hist)
	print('# hsum =', hsum)
	for f in hist:
		output.append(256.0 * f / hsum)
	#next f
	
	return output
#end def HistogramProbability(hist)

def unicodeHistogram(data):
	"""
	Histogram of a unicode string using ord, returns a dict because ord(unicode_value) could return a very large number.
	
	"""
	d = {}
	
	# TODO: I could use a defaultdict here.
	for c in data:
		e = ord(c)
		if e in d.keys():
			d[e] += 1
		else:
			d[e] = 1
		#end if
	#next c
	
	return d
#end def unicodeHistogram(data)

def validateHistogram(hist):
	"""
	Make sure that a histogram of 1M members has an even distribution.
	"""
	# TODO: Standard deviation of a histogram.
	return True
#end def validateHistogram(hist)

def main():
	filename = '/dev/stdin'
	probability = False
	if '-h' in argv:
		print("Usage: python histogram1.py [-h] [-p] filename.bin")
		return
	#end if
	if '-p' in argv:
		probability = True
		argv.pop(argv.index('-p'))
	#end if
	if len(argv) >= 2:
		filename = argv[1]
	#end if
	print("# R0xx0ring", filename)

	# Works on huge files
	a = open(filename, 'rb')
	h = []
	b = b'#'
	while b != b'':
		# Smart to not read the whole file at once it could be large.
		b = a.read(1024)
		h = bufferBinaryHistogram(b, h)
	#loop
	a.close()
	
	if probability:
		# Get the probability if the user passes -p
		p = HistogramProbability(h)
		i = 0
		for f in p:
			print(i, f)
			i += 1
		#next f
	else:
		# Default, print the histogram
		i = 0
		for f in h:
			print(i, f)
			i += 1
		#next f
	#end if
#end def main()

if __name__ == '__main__':
	main()
#end if
